#!/usr/bin/python

#	Copyright 2012 Jon Howell <jonh@jonh.net>
#
#    This file is part of shotgo.
#
#    shotgo is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    shotgo is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with shotgo.  If not, see <http://www.gnu.org/licenses/>.

class Schema:
	def __init__(self, table_name, column_descriptors, name_idx, id_automatic=True):
		self.table_name = table_name
		self.id_automatic = id_automatic
		self.column_descriptors = column_descriptors
		self.column_names = map(lambda cd: cd[name_idx], column_descriptors)
		self._column_map = {}
		for ci in range(len(self.column_names)):
			self._column_map[self.column_names[ci]] = ci

	def __cmp__(self, other):
		i = cmp(self.table_name, other.table_name)
		if (i!=0): return i
		return cmp(self.column_descriptors, other.column_descriptors)

	def __len__(self):
		return len(self.column_names)

	def idx(self, name):
		return self._column_map[name]

	def column_idx_is_automatic(self, idx):
		if (self.id_automatic and idx==0):
			return True
		else:
			return False

class Row:
	def __init__(self, schema, row_data = None):
		self.schema = schema
		if (row_data == None):
			row_data = map(lambda d: None, schema.column_descriptors)
		self.row_data = list(row_data)
		assert(len(row_data)==len(schema.column_descriptors))

	def __getitem__(self, key):
		idx = self.schema.idx(key)
		return self.row_data[idx]
		
	def __setitem__(self, key, value):
		idx = self.schema.idx(key)
		self.row_data[idx] = value

class SQLVerbatim:
	def verbatim(self): raise Exception("abstract")

class SQLNow(SQLVerbatim):
	def verbatim(self): return "NOW()"

class DB:
	def __init__(self, db_name):
		self.db_name = db_name

	def fetchall(self, cur, result): raise Exception("abstract")
		
	def sql_get_unique(self, cur, query, args):
		# finds the unique row that matches the query, or None
		result = self.fetchall(cur, cur.execute(query, args))
		if (len(result) > 1):
			print "%s hits for %s %s" % (len(result), query, args)
			assert(False)
		if (len(result)==0):
			return None
		else:
			return result[0]

	def sql_get_unique_singleton(self, cur, query, args):
		# assumes the unique row is from a query over a single column,
		# and returns just that column
		result = self.sql_get_unique(cur, query, args)
		if (result==None):
			return None
		assert(len(result)==1)
		return result[0]

	def sql_placeholder(self): raise Exception("abstract")

	def insert_row(self, cur, row):
		columns = []
		placeholders = []
		values = []
		for i in range(len(row.schema.column_names)):
			if (row.schema.column_idx_is_automatic(i)):
				continue

			columns.append(row.schema.column_names[i])
			datum = row.row_data[i]
			if (isinstance(datum, SQLVerbatim)):
				placeholders.append(datum.verbatim())
			else:
				placeholders.append(self.sql_placeholder())
				values.append(datum)
		query = "insert into %s(%s) values(%s)" % (
			row.schema.table_name,
			",".join(columns),
			",".join(placeholders))
		values = tuple(values)

#		print "raw query is %s" % query
#		print "insert query is %s" % (query % values)
		cur.execute(query, values)

